"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getMin = exports.getDepositData = exports.getCreateExtrinsic = exports.createExtrinsicEventHandler = void 0;
const xcm_config_1 = require("@moonbeam-network/xcm-config");
const util_1 = require("@polkadot/util");
const polkadot_1 = require("../polkadot");
const sdk_interfaces_1 = require("./sdk.interfaces");
function createExtrinsicEventHandler(config, cb) {
    return ({ events = [], status, txHash }) => {
        const hash = txHash.toHex();
        if (status.isReady) {
            cb({ status: sdk_interfaces_1.ExtrinsicStatus.Sent, txHash: hash });
        }
        if (status.isInBlock) {
            const block = status.asInBlock.toString();
            events.forEach(({ event: { data, method, section } }) => {
                if (section === config.extrinsic.pallet &&
                    method === config.extrinsic.successEvent) {
                    if (method === xcm_config_1.PolkadotXcmExtrinsicSuccessEvent.Attempted) {
                        const eventData = data.at(0);
                        if (eventData.isIncomplete) {
                            cb({
                                status: sdk_interfaces_1.ExtrinsicStatus.Failed,
                                blockHash: block,
                                txHash: hash,
                                message: eventData.asIncomplete.toHuman().join('; '),
                            });
                            return;
                        }
                    }
                    cb({
                        status: sdk_interfaces_1.ExtrinsicStatus.Success,
                        blockHash: block,
                        txHash: hash,
                    });
                }
                if (section === 'system' && method === 'ExtrinsicFailed') {
                    cb({
                        status: sdk_interfaces_1.ExtrinsicStatus.Failed,
                        blockHash: block,
                        txHash: hash,
                        message: data.join('; '),
                    });
                }
            });
        }
    };
}
exports.createExtrinsicEventHandler = createExtrinsicEventHandler;
function getCreateExtrinsic({ account, config, foreignPolkadot, nativeAsset, polkadot, primaryAccount, }) {
    return async (amount) => {
        const fee = (0, xcm_config_1.isMultiCurrency)(config.extrinsic)
            ? await polkadot.getAssetFee(nativeAsset.id, config.origin.weight)
            : 0n;
        return foreignPolkadot.getXcmExtrinsic(account, amount, config.extrinsic, fee, primaryAccount);
    };
}
exports.getCreateExtrinsic = getCreateExtrinsic;
async function getDepositData({ account, asset, config, foreignPolkadot, moonChain, nativeAsset, origin, polkadot, polkadotSigner, primaryAccount, sourceAccount, }) {
    const sourceAddress = typeof sourceAccount === 'string' ? sourceAccount : sourceAccount.address;
    const meta = foreignPolkadot.getMetadata();
    const createExtrinsic = getCreateExtrinsic({
        account,
        config,
        foreignPolkadot,
        nativeAsset,
        polkadot,
        primaryAccount,
    });
    const [decimals, sourceBalance, existentialDeposit, sourceFeeBalance, moonChainFee, sourceMinBalance = 0n, min,] = await Promise.all([
        polkadot.getAssetDecimals(asset.id),
        foreignPolkadot.getGenericBalance(primaryAccount || sourceAddress, config.balance),
        foreignPolkadot.getExistentialDeposit(),
        config.sourceFeeBalance
            ? foreignPolkadot.getGenericBalance(sourceAddress, config.sourceFeeBalance)
            : undefined,
        config.isNativeAssetPayingMoonFee
            ? polkadot.getAssetFee(nativeAsset.id, config.origin.weight)
            : undefined,
        config.sourceMinBalance
            ? foreignPolkadot.getAssetMinBalance(config.sourceMinBalance)
            : undefined,
        // eslint-disable-next-line @typescript-eslint/no-use-before-define
        getMin({ asset, config, moonChain, polkadot }),
    ]);
    return {
        asset: { ...asset, decimals },
        existentialDeposit,
        min,
        moonChainFee,
        native: { ...nativeAsset, decimals: meta.decimals },
        origin,
        source: config.origin,
        sourceBalance,
        sourceFeeBalance: !(0, util_1.isUndefined)(sourceFeeBalance)
            ? { ...meta, balance: sourceFeeBalance }
            : undefined,
        sourceMinBalance,
        getFee: async (amount = sourceBalance) => {
            const extrinsic = await createExtrinsic(amount);
            const info = await extrinsic.paymentInfo(sourceAccount);
            return info.partialFee.toBigInt();
        },
        send: async (amount, cb) => {
            const extrinsic = await createExtrinsic(amount);
            const hash = await extrinsic.signAndSend(sourceAccount, {
                signer: polkadotSigner,
            }, cb && createExtrinsicEventHandler(config, cb));
            return hash.toString();
        },
    };
}
exports.getDepositData = getDepositData;
async function getMin({ asset, config, moonChain, polkadot, }) {
    if (config.sourceFeeBalance && config.isNativeAssetPayingMoonFee) {
        return 0n;
    }
    return asset.isNative
        ? polkadot_1.PolkadotService.getChainMin(config.origin.weight, moonChain.unitsPerSecond)
        : polkadot.getAssetFee(asset.id, config.origin.weight);
}
exports.getMin = getMin;
//# sourceMappingURL=sdk.deposit.js.map