"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.subscribeToAssetsBalanceInfo = exports.sortByBalanceAndChainName = exports.toDecimal = exports.isXcmSdkWithdraw = exports.isXcmSdkDeposit = void 0;
function isXcmSdkDeposit(config) {
    return !!config.from;
}
exports.isXcmSdkDeposit = isXcmSdkDeposit;
function isXcmSdkWithdraw(config) {
    return !!config.to;
}
exports.isXcmSdkWithdraw = isXcmSdkWithdraw;
function toDecimal(number, decimals, maxDecimal = 6) {
    const decimalMultiplier = 10 ** maxDecimal;
    const dividend = number * BigInt(decimalMultiplier);
    const divisor = 10n ** BigInt(decimals);
    return Number(dividend / divisor) / decimalMultiplier;
}
exports.toDecimal = toDecimal;
function sortByBalanceAndChainName(a, b) {
    if (a.asset.isNative) {
        return -1;
    }
    if (b.asset.isNative) {
        return 1;
    }
    const aDecimal = toDecimal(a.balance.balance, a.balance.decimals);
    const bDecimal = toDecimal(b.balance.balance, b.balance.decimals);
    if (aDecimal || bDecimal) {
        return bDecimal - aDecimal;
    }
    const aName = (a.origin.name + a.asset.originSymbol).toLowerCase();
    const bName = (b.origin.name + b.asset.originSymbol).toLowerCase();
    if (aName < bName) {
        return -1;
    }
    if (aName > bName) {
        return 1;
    }
    return 0;
}
exports.sortByBalanceAndChainName = sortByBalanceAndChainName;
async function subscribeToAssetsBalanceInfo({ account, polkadot, configBuilder, cb, }) {
    let lastBalance = 0n;
    let lastInfo = [];
    const handler = (data) => {
        const isBalance = typeof data === 'bigint';
        lastBalance = isBalance ? data : lastBalance;
        lastInfo = (isBalance ? lastInfo : data)
            .map((info) => {
            if (info.asset.isNative) {
                // eslint-disable-next-line no-param-reassign
                info.balance.balance = lastBalance;
            }
            return info;
        })
            .sort(sortByBalanceAndChainName);
        if (lastInfo.length) {
            cb(lastInfo);
        }
    };
    const unsubscribeBalance = await polkadot.subscribeToBalance(account, handler);
    const unsubscribeInfo = await polkadot.subscribeToAssetsBalanceInfo(account, configBuilder, handler);
    return () => {
        unsubscribeBalance();
        unsubscribeInfo();
    };
}
exports.subscribeToAssetsBalanceInfo = subscribeToAssetsBalanceInfo;
//# sourceMappingURL=sdk.utils.js.map